<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use DB;
use Storage;
use Validator;

class ProjectController extends Controller
{
    private $table = "projects";

    public function detail()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $client = $this->auth_client();
        $id = request()->id ?? 0;
        $search = request()->search ?? "";

        $time_zone = request()->time_zone ?? "";
        if (!empty($time_zone))
        {
            date_default_timezone_set($time_zone);
        }

        $project = DB::table($this->table)
            ->where("id", "=", $id)
            ->where("client_id", "=", $client->id)
            ->whereNull("deleted_at")
            ->first();

        if ($project == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project not found."
            ]);
        }

        $obj = [
            "id" => $project->id ?? 0,
            "name" => $project->name ?? "",
        ];

        $tasks = DB::table("tasks")
            ->where("client_id", "=", $client->id)
            ->where("project_id", "=", $project->id);

        if (!empty($search))
        {
            $projects = $projects->where(function ($query) use ($search) {
                $query->where("title", "LIKE", "%" . $search . "%")
                    ->orWhere("description", "LIKE", "%" . $search . "%")
                    ->orWhere("status", "LIKE", "%" . $search . "%")
                    ->orWhere("payment_status", "LIKE", "%" . $search . "%");
            });
        }

        $tasks = $tasks->whereNull("deleted_at")
            ->paginate(50);

        $tasks_arr = [];
        foreach ($tasks as $task)
        {
            $task_obj = [
                "id" => $task->id ?? 0,
                "title" => $task->title ?? "",
                "description" => $task->description ?? "",
                "price" => $task->price ?? 0,
                "status" => $task->status ?? "",
                "payment_status" => $task->payment_status ?? "",
                "updated_at" => date("d M, Y h:i:s a", strtotime($task->updated_at . " UTC"))
            ];

            $task_obj["payment_status_str"] = $this->capitalize(str_replace("_", " ", $task_obj["payment_status"]));
            array_push($tasks_arr, (object) $task_obj);
        }

        return response()->json([
            "status" => "success",
            "message" => "Data has been fetched.",
            "project" => (object) $obj,
            "tasks" => $tasks_arr,
            "pages" => $tasks->lastPage()
        ]);
    }

    public function single()
    {
        $id = request()->id ?? 0;

        return view("projects/single", [
            "id" => $id
        ]);
    }

    public function fetch()
    {
        $client = $this->auth_client();
        $search = request()->search ?? "";

        $time_zone = request()->time_zone ?? "";
        if (!empty($time_zone))
        {
            date_default_timezone_set($time_zone);
        }

        $projects = DB::table($this->table)
            ->select($this->table . ".*")
            ->where("client_id", "=", $client->id);
        if (!empty($search))
        {
            $table = $this->table;
            $projects = $projects->where(function ($query) use ($search, $table) {
                $query->where($table . ".name", "LIKE", "%" . $search . "%");
            });
        }

        $projects = $projects->whereNull($this->table . ".deleted_at")
            ->orderBy($this->table . ".id", "desc")
            ->paginate(50);

        $projects_arr = [];
        foreach ($projects as $project)
        {
            $obj = [
                "id" => $project->id ?? 0,
                "name" => $project->name ?? "",
                "created_at" => date("d M, Y h:i:s a", strtotime($project->created_at . " UTC")),
                "updated_at" => date("d M, Y h:i:s a", strtotime($project->updated_at . " UTC"))
            ];

            array_push($projects_arr, (object) $obj);
        }

        return response()->json([
            "status" => "success",
            "message" => "Data has been fetched.",
            "projects" => $projects_arr,
            "pages" => $projects->lastPage()
        ]);
    }

    public function destroy()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $id = request()->id ?? 0;

        $project = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($project == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project not found."
            ]);
        }

        DB::table($this->table)
            ->where("id", "=", $project->id)
            ->update([
                "deleted_at" => now()->utc()
            ]);

        return response()->json([
            "status" => "success",
            "message" => "Project has been deleted."
        ]);
    }

    public function update()
    {
        $validator = Validator::make(request()->all(), [
            "id" => "required",
            "name" => "required",
            "client_id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $id = request()->id ?? 0;
        $name = request()->name ?? "";
        $client_id = request()->client_id ?? 0;

        $project = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($project == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project not found."
            ]);
        }

        $project_id = $project->id;

        $client = DB::table("clients")
            ->where("id", "=", $client_id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client not found."
            ]);
        }

        $exists_by = "";
        $project = DB::table($this->table);
        if (!empty($name))
        {
            $project = $project->where("name", "=", $name)
                ->where("id", "!=", $project_id);
            $exists_by = "name";
        }
        $project = $project->whereNull("deleted_at")->first();

        if ($project != null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project with same " . $exists_by . " already exists."
            ]);
        }

        $obj = [
            "name" => $name,
            "client_id" => $client->id,
            "updated_at" => now()->utc()
        ];

        DB::table($this->table)
            ->where("id", "=", $project_id)
            ->update($obj);

        return response()->json([
            "status" => "success",
            "message" => "Data has been updated.",
            "project" => (object) $obj
        ]);
    }

    public function edit()
    {
        $id = request()->id ?? 0;

        $project = DB::table($this->table)
            ->where("id", "=", $id)
            ->whereNull("deleted_at")
            ->first();

        if ($project == null)
        {
            abort(404);
        }

        $clients = DB::table("clients")
            ->whereNull("deleted_at")
            ->orderBy("name", "asc")
            ->get();

        return view("admin/projects/edit", [
            "project" => $project,
            "clients" => $clients
        ]);
    }

    public function fetch_admin()
    {
        $this->admin_auth();
        $search = request()->search ?? "";

        $time_zone = $_POST["time_zone"] ?? "";
        if (!empty($time_zone))
        {
            date_default_timezone_set($time_zone);
        }

        $projects = DB::table($this->table)
            ->select($this->table . ".*", "clients.name AS client_name",
                "clients.email AS client_email", "clients.phone AS client_phone")
            ->join("clients", "clients.id", "=", $this->table . ".client_id");
        if (!empty($search))
        {
            $table = $this->table;
            $projects = $projects->where(function ($query) use ($search, $table) {
                $query->where($table . ".name", "LIKE", "%" . $search . "%");
            });
        }

        $projects = $projects->whereNull($this->table . ".deleted_at")
            ->orderBy($this->table . ".id", "desc")
            ->paginate(50);

        $projects_arr = [];
        foreach ($projects as $project)
        {
            $obj = [
                "id" => $project->id ?? 0,
                "name" => $project->name ?? "",
                "client" => (object) [
                    "id" => $project->client_id ?? 0,
                    "name" => $project->client_name ?? "",
                    "email" => $project->client_email ?? "",
                    "phone" => $project->client_phone ?? ""
                ],
                "created_at" => date("d M, Y h:i:s a", strtotime($project->created_at . " UTC")),
                "updated_at" => date("d M, Y h:i:s a", strtotime($project->updated_at . " UTC"))
            ];

            array_push($projects_arr, (object) $obj);
        }

        return response()->json([
            "status" => "success",
            "message" => "Data has been fetched.",
            "projects" => $projects_arr,
            "pages" => $projects->lastPage()
        ]);
    }

    public function store()
    {
        $validator = Validator::make(request()->all(), [
            "name" => "required",
            "client_id" => "required"
        ]);

        if (!$validator->passes() && count($validator->errors()->all()) > 0)
        {
            return response()->json([
                "status" => "error",
                "message" => $validator->errors()->all()[0]
            ]);
        }

        $this->admin_auth();
        $name = request()->name ?? "";
        $client_id = request()->client_id ?? 0;

        $client = DB::table("clients")
            ->where("id", "=", $client_id)
            ->whereNull("deleted_at")
            ->first();

        if ($client == null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Client not found."
            ]);
        }

        $exists_by = "";
        $project = DB::table($this->table);
        if (!empty($name))
        {
            $project = $project->where("name", "=", $name);
            $exists_by = "name";
        }
        $project = $project->whereNull("deleted_at")->first();

        if ($project != null)
        {
            return response()->json([
                "status" => "error",
                "message" => "Project with same " . $exists_by . " already exists."
            ]);
        }

        $obj = [
            "name" => $name,
            "client_id" => $client->id,
            "created_at" => now()->utc(),
            "updated_at" => now()->utc()
        ];

        $obj["id"] = DB::table($this->table)
            ->insertGetId($obj);

        return response()->json([
            "status" => "success",
            "message" => "Data has been added.",
            "project" => (object) $obj
        ]);
    }

    public function add()
    {
        $clients = DB::table("clients")
            ->whereNull("deleted_at")
            ->orderBy("name", "asc")
            ->get();

        return view("admin/projects/add", [
            "clients" => $clients
        ]);
    }

    public function admin_index()
    {
        return view("admin/projects/index");
    }
}
